---
gitea: none
include_toc: true
---

## Current architecture 

> **Note:**\
> The ingress port mapping uses both port `80` and port `443`.
```yaml
Public ingress port mapping: 192.168.1.2
```

### Standalone

```yaml
Gateway: 192.168.1.1
Pi4: 192.168.1.2
Srv: 192.168.1.3
```

### Kluster

> Kubernetes Cluster

A set of Orange PI 5, so far all of them are the 4GB of RAM version.

```yaml
Masterk: 192.168.1.10
Slave01: 192.168.1.11
```

## Objectives

- Set the Kubernetes Istio public Ingress as the main ingress.
- Join the Pi4 to the Kubernetes Cluster, therefore the Pi4/cluster cannot run dependencies to the same cluster. Pi4 will be the control plane  
- Logs should be exported to the host `srv`.
- Services in kubernetes should have a health probe and prometheus configured to allow for metrics. 
- Allow to set some CI/CD pipelines -> automatically build and deploy some images that I manage/own. (Not done on this example.) 

## Requirements

- NFS server for PVC on Kubernetes.
- Check for OWASP security
- Public and Locale Ingress.
- Configure a Cert Manager to provide public certs.
> https://medium.com/@rd.petrusek/kubernetes-istio-cert-manager-and-lets-encrypt-c3e0822a3aaf

- Pi4 USB powered storage.
- Normalize `arm7` running distribution -> choose one OS that that works on all devices.

- New `Gitea` account with limited permissions for CI/CD.
- Istio Load Balancer deployed must maintain a static IP.

## Dependencies

- DHCP service
- DNS service
- Gitea service (and it's database)

## Suggestions/notes.

- Instead of changing the port tunnel IP from the router, could set the Istio Ingress to have the current Pi4 IP, yet this would limit some process.  
 
- Performing this without "downtime" will be extremely annoying, since if I need/want to use the Pi4 as the control plane device, the cluster itself should need to be recreated, therefore, until the Pi4 is not removed from the equation, the cluster cannot be created/used/enabled. Tips:
  - First move the key services to the SRV.
  - Remove the current drive from the Pi4, create an Ingress on the current Kubernetes Cluster with the IP set to the Pi4, configure Certificate Manager temporarily thereto allow traffic meanwhile the Pi4 is no longer available.
  - Explore storage options for the Pi4, aka try the low voltage/energy consumption m.2 drive bought recently.  

- Another option would be, moving everything to the SRV and use that meanwhile, this would allow for less downtime, and I could get to do whatever I want with the Pi4 and Cluster in the meantime, could deploy the `Traefik` with the Pi4 IP (.2) to allow for *shooting down the Pi4 shenanigans*.

### Regarding the Cert-Manager

> https://cert-manager.io/

`HTTP01 challenge`, doesn't allow for wildcard certificates, therefore, for better comfort, use the `DNS01 challange`.

Current Issue? For X and y, I need to wait for a while for the DNS provider to release the domain and acquire it back again.

### Core services

- Tube
 
- Jellyfin

- Gitea

- Filebrowser

- qBitTorrent (accessible locally)

## Limitations

- Limited hardware

- Only 1 external IP / router, therefore I need to do some Cert Manager tests before.

## Pi4 Storage related-specific

- If I keep using a SSD, 3D print a base for it, the Pi4 doesn't need a shell as it already has one, but just a platform that allows the Pi4 to sit on the top.  

## Procedure

> Note:\
> I will be making backups of certain configurations before modifying them, yet I won't be displaying it for the sake of reducing a bit the amount of text dumped here.

### Part 1

> Completed 19/July/2023

- [x] Move local DHCP and DNS to the host `srv`.

- [x] Move Gitea to the host `srv`.

- [ ] Move registry to the host `srv`. -> Not relevant for my current setup, therefore can and (probably will) be skipped.

- [X] Update Traefik ingress configs

#### Rollback plan

> Not required.

- Launch the services in the `Pi4` server.

- Return the Traefik configuration to the previous one.

- Modify the DNS configurations (if applicable).

### Part 2

[//]: # (> Started 19/July/2023)

[//]: # (> Finished /July/2023)

> "Completed" 24/July/2023\
> Last update 26/July/2023

- [x] Deploy an Istio Ingress Gateway to allow access to the "core" services.

- [x] Deploy an Istio Egress Gateway to allow egress towards the "core" local services.

- [x] Deploy an Ingress LB for local thingies.

> **Note:**\
> - https://istio.io/latest/docs/tasks/traffic-management/egress/ 
> - https://istio.io/latest/docs/tasks/traffic-management/egress/egress-kubernetes-services/ 
> - https://istio.io/latest/docs/reference/config/istio.operator.v1alpha1/ 
> - https://istio.io/latest/docs/reference/config/istio.mesh.v1alpha1/#MeshConfig 
> - https://istio.io/latest/docs/setup/additional-setup/config-profiles/ 
> - https://istio.io/latest/docs/setup/additional-setup/customize-installation/

### Part 3

> "Completed" 24/July/2023\
> Last update 26/July/2023

- [x] Configure the public Istio Ingress to route traffic towards Gitea, Jellyfin, Filebrowser and Tube services.

- [x] Deploy Certificate Manager on the current `Kluster` with `Let's Encrypt` certificate provisioning.

- [x] Update the router `NAT - Virtual Servers` to set the Istio Ingress deployed as the new Public Ingress.

- [x] Ensure the Certificates are being provisioned.

- [x] Backup the Istio/Kubernetes configuration files used so far.

- [x] Deploy configurations to route some services through the Istio `istio-local-ingress` Load Balancer deployed.

> **Note**:\
> Regarding Let's Encrypt certificate provisioning, for testing purposes the `staging` environment should be used, nevertheless on my scenario I am running directly on the production environment, why?\
> - `Staging` and `Production` behave different, therefore one can get the certificates verified on `Staging` and not on `Production`.
> - I ran into some issues regarding the sentence from above, so there was some back and forth, this topic is mentioned [here at the end.](#2x1-able-to-get-staging-le-certs-but-not-the-production-one-when-using-custom-istio-selector-and-only-able-to-get-production-le-certs-when-using-the-default-istio-ingressgateway-selector)
> - Since there was "back and forth" I sort of cheated and set this as it is.

> **Additional Note**:\
> The "local" gateway will have a TLS secret that won't be created until the next part.

#### Rollback plan

- Update the router `Virtual Service Port Mapping` to set back the Traefik Ingress form the Pi4 host.
- Delete the deployed configurations.

### Part 4

> Completed 27/July/2023\

- [x] Deploy locally a Certificate Authorization Service (on the SRV host.)

- [x] Update local DNS records accordingly.

> **Notes**:\
> Since the `VirtualService` files from Part 

> **Additional Notes**:
> - https://www.reddit.com/r/selfhosted/comments/owplv5/any_self_hosted_certificate_authority/
> - https://github.com/minio/certgen
> - https://github.com/rabbitmq/tls-gen
> - https://smallstep.com/blog/private-acme-server/
> - https://hub.docker.com/r/smallstep/step-ca
> - https://smallstep.com/docs/tutorials/kubernetes-acme-ca/
> - https://smallstep.com/blog/automate-docker-ssl-tls-certificates/
> - https://systemweakness.com/create-internal-ssl-certificates-with-cert-manager-851fc886628e


#### Rollback plan

- As much, delete the deployed configurations.

### Part 5

- [ ] Explore Pi4 Storage options.

- [ ] Consider Storage options for the OrangePi5.

#### Rollback plan

- Return the acquired drives to Amazon?

### Part 6

- [ ] ~~Wipe~~ (**don't wipe** just use a different drive) and recreate the current `Kluster`, this time using the Pi4 as a _master_, and the 2 Orange Pi5 as _slaves_ (this will require updating the DNS/DHCP local services).

> **Note**:\
> I can make a new cluster on the Pi4, and remove the taint that prevents from scheduling pods on that node. Deploy everything inside (a well a LB with the same exact IP than the current one, and proceed to stop the Orange PI 5), then "reformat" the OPi5s with a new distro, install stuff etc., and join them to the cluster running on the Pi4. 

> **Notes:**
> - https://istio.io/latest/docs/setup/platform-setup/prerequisites/
> - https://istio.io/latest/docs/ops/deployment/requirements/

### Part 7

- [ ] Deploy NFS service on the `media SRV` host.


### Part 8

- [ ] Deploy Istio security.

### Part 10

- [ ] Update the `Current Setup` documentation with the new container and architecture rearrangement. 

- [ ] Migrate some lightweight/not data heavy services from the `media SRV` to the `Kluster`.

- [ ] Update the `Current Setup` documentation with the new container and architecture rearrangement.

#### Rollback plan

- Revert orange Pi 5 to the previous drive.

- Revert Pi4 to the previous drive.

- Delete the NFS service deployed.

- Run the old migrated services back on the `media SRV` host.

### Part 11

- Set wildcards certificates through `ACME DNS01` challenge.

### Extras?

#### Horizontal Pod Autoscaling for the Istio LBs. 

# Execution

## Part 1
### Transfer local network dependencies services from Pi4 to SRV.

#### Install ZIP on `Pi4`

```shell
dnf install zip
```
```text
Last metadata expiration check: 1:17:47 ago on Wed 19 Jul 2023 07:26:42 AM UTC.
Dependencies resolved.
==========================================================================================================================================================================================================================================
 Package                                               Architecture                                            Version                                                       Repository                                              Size
==========================================================================================================================================================================================================================================
Installing:
 zip                                                   aarch64                                                 3.0-23.el8                                                    baseos                                                 264 k
Installing dependencies:
 unzip                                                 aarch64                                                 6.0-46.el8                                                    baseos                                                 188 k

Transaction Summary
==========================================================================================================================================================================================================================================
Install  2 Packages

Total download size: 452 k
Installed size: 1.4 M
Is this ok [y/N]: y
Downloading Packages:
(1/2): zip-3.0-23.el8.aarch64.rpm                                                                                                                                                                         1.7 MB/s | 264 kB     00:00    
(2/2): unzip-6.0-46.el8.aarch64.rpm                                                                                                                                                                       1.2 MB/s | 188 kB     00:00    
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Total                                                                                                                                                                                                     790 kB/s | 452 kB     00:00     
Running transaction check
Transaction check succeeded.
Running transaction test
Transaction test succeeded.
Running transaction
  Preparing        :                                                                                                                                                                                                                  1/1 
  Installing       : unzip-6.0-46.el8.aarch64                                                                                                                                                                                         1/2 
  Installing       : zip-3.0-23.el8.aarch64                                                                                                                                                                                           2/2 
  Running scriptlet: zip-3.0-23.el8.aarch64                                                                                                                                                                                           2/2 
  Verifying        : unzip-6.0-46.el8.aarch64                                                                                                                                                                                         1/2 
  Verifying        : zip-3.0-23.el8.aarch64                                                                                                                                                                                           2/2 

Installed:
  unzip-6.0-46.el8.aarch64                                                                                             zip-3.0-23.el8.aarch64                                                                                            

Complete!
```

#### DHCP

##### From Pi4

###### Compress DHCP service

```shell
zip -r /tmp/dhcpd.zip /scripts/docker/dhcpd
```

```text
  adding: scripts/docker/dhcpd/dhcpd.leases (stored 0%)
  adding: scripts/docker/dhcpd/docker-compose.yaml (deflated 55%)
  adding: scripts/docker/dhcpd/dhcpd.conf (deflated 61%)
  adding: scripts/docker/dhcpd/Dockerfile (deflated 61%)
```

###### List compressed file

```shell
ls /tmp/dhcpd.zip  -l
```

```text
-rw-r--r--. 1 root root 1987 Jul 19 09:00 /tmp/dhcpd.zip
```

###### Send file to the srv.

```shell
scp /scripts/docker/dhcpd.zip user@srv:/tmp
```

##### From SRV

###### Unzip to the desired location, maintaining permissions.

```shell
unzip -X /tmp/dhcpd.zip -d /scripts/docker/
```

```text
Archive:  /tmp/dhcpd.zip
   creating: /scripts/docker/scripts/docker/dhcpd/
 extracting: /scripts/docker/scripts/docker/dhcpd/dhcpd.leases  
  inflating: /scripts/docker/scripts/docker/dhcpd/docker-compose.yaml  
  inflating: /scripts/docker/scripts/docker/dhcpd/dhcpd.conf  
  inflating: /scripts/docker/scripts/docker/dhcpd/Dockerfile
```

###### Modify the DHCP configuration to point to this server as a DNS Server.

```shell
nano /scripts/docker/dhcpd/dhcpd.conf
```

Replace

```text
  option domain-name-servers 192.168.1.2, 9.9.9.9, 149.112.112.112; # IBM Quad9
```

For

```text
  option domain-name-servers 192.168.1.3, 9.9.9.9, 149.112.112.112; # IBM Quad9
```

###### Fix the DHCP interface to listen.

```shell
nano /scripts/docker/dhcpd/Dockerfile
```

Replace
```text
                        "-group", "dhcp", "eth0"]
```

For

```text
                        "-group", "dhcp", "enp3s0"]
```


#### DNS

##### From Pi4

###### Compress DHCP service

```shell
zip -r /tmp/coredns.zip /scripts/docker/coredns
```

```text
  adding: scripts/docker/coredns/ (stored 0%)
  adding: scripts/docker/coredns/config/ (stored 0%)
  adding: scripts/docker/coredns/config/Corefile (deflated 31%)
  adding: scripts/docker/coredns/config/example.db (deflated 37%)
  adding: scripts/docker/coredns/config/filter.home.db (deflated 71%)
  adding: scripts/docker/coredns/docker-compose.yml (deflated 57%)
```

###### List compressed file

```shell
ls /tmp/coredns.zip  -l
```

```text
-rw-r--r--. 1 root root 2284 Jul 19 09:34 /tmp/coredns.zip
```

###### Send file to the srv.

```shell
scp /tmp/coredns.zip user@srv:/tmp
```

##### From SRV

###### Unzip to the desired location, maintaining permissions.

```shell
unzip -X /tmp/coredns.zip -d /scripts/docker/
```

```text
   creating: /scripts/docker/scripts/docker/coredns/
   creating: /scripts/docker/scripts/docker/coredns/config/
  inflating: /scripts/docker/scripts/docker/coredns/config/Corefile  
  inflating: /scripts/docker/scripts/docker/coredns/config/example.db  
  inflating: /scripts/docker/scripts/docker/coredns/config/filter.home.db  
  inflating: /scripts/docker/scripts/docker/coredns/docker-compose.yml
```

###### Modify the DHCP configuration to point to this server as a DNS Server.

```shell
nano /scripts/docker/scripts/docker/coredns/config/filter.home.db 
```

Replace

```text
dhcp    IN	CNAME   pi4
dns     IN	CNAME   pi4
```

For

```text
dhcp    IN	CNAME   srv
dns     IN	CNAME   srv
```


##### Realize that things uncompressed into `/scripts/docker/scripts/docker/`, fix that.

Use the `mv` command to fix it.

Remove the folders with `rmdir` to avoid surprises.

### Update nameservers
#### From SRV

##### Start the DNS service

```shell
cd /scripts/docker/coredns/ && docker-compose up -d
```
```text
[+] Running 3/3
⠿ coredns Pulled                                                                                                                                                                                                                    3.1s
⠿ 25b7032c281a Pull complete                                                                                                                                                                                                      0.5s
⠿ 3799eae1a077 Pull complete                                                                                                                                                                                                      1.1s
[+] Running 2/2
⠿ Network coredns_default  Created                                                                                                                                                                                                  0.2s
⠿ Container coredns        Started                                                                                                                                                                                                  0.5s
```

```shell
docker-compose logs -f
```

```text
coredns  | .:53
coredns  | filter.home.:53
coredns  | CoreDNS-1.10.1
coredns  | linux/amd64, go1.20, 055b2c3
```

###### Test the DNS service

From any host within the network that can reach / is supposed to use that DNS server:

```shell
dig google.es @192.168.1.3
```

```text
; <<>> DiG 9.18.16 <<>> google.es @192.168.1.3
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 49583
;; flags: qr rd ra; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags:; udp: 1232
;; QUESTION SECTION:
;google.es.			IN	A

;; ANSWER SECTION:
google.es.		99	IN	A	142.250.200.67

;; Query time: 13 msec
;; SERVER: 192.168.1.3#53(192.168.1.3) (UDP)
;; WHEN: Wed Jul 19 12:30:42 CEST 2023
;; MSG SIZE  rcvd: 63
```

[//]: # (##### Restart DNS service _just in case_)

[//]: # ()
[//]: # (```shell)

[//]: # (cd /scripts/docker/coredns/ && docker-compose restart)

[//]: # (```)

[//]: # (```text)

[//]: # ([+] Running 1/1)

[//]: # ( ⠿ Container coredns  Started                                                                                                                                                                                                        0.7s                                                                                                                                                                                              0.5s)

[//]: # (```)

#### Start the DHCP Service

###### Set static IP on the desired interface (enp3s0 in my case)

```text
nano  /etc/sysconfig/network-scripts/ifcfg-enp3s0
```

```text
# Network type
TYPE=Ethernet
# Device Name #
NAME=enp3s0
DEVICE=enp3s0
# Static IP Address #
PROXY_METHOD=none
BROWSER_ONLY=no
BOOTPROTO=none
# Server IP #
IPADDR=192.168.1.3
# DNS SERVERS #
DNS1=127.0.0.1
DNS2=8.8.8.8
# GATEWAY ADDRESS #
GATEWAY=192.168.1.1
# SUBNET MASK #
SUBNET MASK=255.255.255.0
DEFROUTE=yes
IPV4_FAILURE_FATAL=no
# Disable ipv6#
IPV6INIT=no

# Activate on system Boot #
ONBOOT=yes
```

###### Restart server to ensure everything is working correctly.

```shell
reboot
```


###### Start container/service

```shell
cd /scripts/docker/dhcpd/ && docker-compose up 
```


<!--suppress SpellCheckingInspection, SpellCheckingInspection -->
<pre>[+] Running 0/1
<span style="color:#FF7F7F"> ⠿ isc_dhcp Error                                                                                                                                                                                                                    1.4s</span>
[+] Building 4.2s (8/8) FINISHED                                                                                                                                                                                                          
<span style="color:#7FBAFF"> =&gt; [internal] load .dockerignore                                                                                                                                                                                                    0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; transferring context: 2B                                                                                                                                                                                                      0.0s</span>
<span style="color:#7FBAFF"> =&gt; [internal] load build definition from Dockerfile                                                                                                                                                                                 0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; transferring dockerfile: 1.03kB                                                                                                                                                                                               0.0s</span>
<span style="color:#7FBAFF"> =&gt; [internal] load metadata for docker.io/library/alpine:latest                                                                                                                                                                     1.5s</span>
<span style="color:#7FBAFF"> =&gt; [1/4] FROM docker.io/library/alpine:latest@sha256:82d1e9d7ed48a7523bdebc18cf6290bdb97b82302a8a9c27d4fe885949ea94d1                                                                                                               0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; resolve docker.io/library/alpine:latest@sha256:82d1e9d7ed48a7523bdebc18cf6290bdb97b82302a8a9c27d4fe885949ea94d1                                                                                                               0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; sha256:82d1e9d7ed48a7523bdebc18cf6290bdb97b82302a8a9c27d4fe885949ea94d1 1.64kB / 1.64kB                                                                                                                                       0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; sha256:25fad2a32ad1f6f510e528448ae1ec69a28ef81916a004d3629874104f8a7f70 528B / 528B                                                                                                                                           0.0s</span>
<span style="color:#7FBAFF"> =&gt; =&gt; sha256:c1aabb73d2339c5ebaa3681de2e9d9c18d57485045a4e311d9f8004bec208d67 1.47kB / 1.47kB                                                                                                                                       0.0s</span>
<span style="color:#7FBAFF"> =&gt; [2/4] RUN apk update &amp;&amp; apk add --no-cache dhcp                                                                                                                                                                                  1.7s</span>
<span style="color:#7FBAFF"> =&gt; [3/4] RUN mkdir /var/lib/dhcpd &amp;&amp; touch /var/lib/dhcpd/dhcpd.leases &amp;&amp; chown dhcp:dhcp /var/lib/dhcpd -R &amp;&amp; mkdir -p /run/dhcp                                                                                                   0.4s</span> 
<span style="color:#7FBAFF"> =&gt; [4/4] RUN id dhcp                                                                                                                                                                                                                0.4s</span> 
<span style="color:#7FBAFF"> =&gt; exporting to image                                                                                                                                                                                                               0.1s</span> 
<span style="color:#7FBAFF"> =&gt; =&gt; exporting layers                                                                                                                                                                                                              0.1s</span> 
<span style="color:#7FBAFF"> =&gt; =&gt; writing image sha256:07b6db561779c547d96d07fe3a995668ba459ac72fe8c5b48de5e8956e138dd9                                                                                                                                         0.0s</span> 
<span style="color:#7FBAFF"> =&gt; =&gt; naming to docker.io/library/rpi_dhcp                                                                                                                                                                                          0.0s</span> 

Use &apos;docker scan&apos; to run Snyk tests against images to find vulnerabilities and learn how to fix them
<span style="color:#7FBAFF">[+] Running 1/1</span>
<span style="color:#7FBAFF"> ⠿ Container dhcpd  Created                                                                                                                                                                                                          0.0s</span>
</pre>



```shell
docker-compose logs -f
```

<pre><span style="color:#7F7FFF">dhcpd  | </span>Internet Systems Consortium DHCP Server 4.4.3-P1
<span style="color:#7F7FFF">dhcpd  | </span>Copyright 2004-2022 Internet Systems Consortium.
<span style="color:#7F7FFF">dhcpd  | </span>All rights reserved.
<span style="color:#7F7FFF">dhcpd  | </span>For info, please visit https://www.isc.org/software/dhcp/
<span style="color:#7F7FFF">dhcpd  | </span>WARNING: Host declarations are global.  They are not limited to the scope you declared them in.
<span style="color:#7F7FFF">dhcpd  | </span>Config file: /etc/dhcp/dhcpd.conf
<span style="color:#7F7FFF">dhcpd  | </span>Database file: /var/lib/dhcpd/dhcpd.leases
<span style="color:#7F7FFF">dhcpd  | </span>PID file: /run/dhcp/dhcpd.pid
<span style="color:#7F7FFF">dhcpd  | </span>Wrote 0 deleted host decls to leases file.
<span style="color:#7F7FFF">dhcpd  | </span>Wrote 0 new dynamic host decls to leases file.
<span style="color:#7F7FFF">dhcpd  | </span>Wrote 0 leases to leases file.
<span style="color:#7F7FFF">dhcpd  | </span>Listening on LPF/enp3s0/50:3e:aa:03:7a:37/192.168.1.0/24
<span style="color:#7F7FFF">dhcpd  | </span>Sending on   LPF/enp3s0/50:3e:aa:03:7a:37/192.168.1.0/24
<span style="color:#7F7FFF">dhcpd  | </span>Sending on   Socket/fallback/fallback-net
<span style="color:#7F7FFF">dhcpd  | </span>Server starting service.
</pre>



#### Update rest of nameservers

Modify the `/etc/resolve.conf` or whatever static configurations you have deployed in specific devices.

On my scenario only applies to the pi4.

### Transfer Gitea from Pi4 to SRV.

#### Service

##### From Pi4

###### Compress Gitea service

```shell
zip -r /tmp/gitea.zip /scripts/docker/gitea
```

```text
zip -r /tmp/gitea.zip /scripts/docker/gitea
  adding: scripts/docker/gitea/ (stored 0%)
  adding: scripts/docker/gitea/app.ini (deflated 47%)
  adding: scripts/docker/gitea/docker-compose.yml (deflated 60%)
  adding: scripts/docker/gitea/renderers/ (stored 0%)
  adding: scripts/docker/gitea/renderers/.placeholder (stored 0%)
  adding: scripts/docker/gitea/Dockerfile (deflated 49%)
```

###### List compressed file

```shell
ls /tmp/gitea.zip  -l
```

```text
-rw-r--r--. 1 root root 2231 Jul 19 10:39 /tmp/gitea.zip
```

###### Send file to the srv.

```shell
scp /tmp/gitea.zip user@srv:/tmp
```

#### From SRV

##### Unzip to the desired location, maintaining permissions.

> Learn from the mistakes, use the `/` path and let everything sit correctly by itself.

```shell
unzip -X /tmp/gitea.zip -d /
```

```text
   creating: /scripts/docker/gitea/
  inflating: /scripts/docker/gitea/app.ini  
  inflating: /scripts/docker/gitea/docker-compose.yml  
   creating: /scripts/docker/gitea/renderers/
 extracting: /scripts/docker/gitea/renderers/.placeholder  
  inflating: /scripts/docker/gitea/Dockerfile
```

#### Volumes data

##### From Pi4

###### Stop the service

```shell
docker-compose down
```

<pre>
<span style="color:#7FBAFF">[+] Running 3/3</span>
<span style="color:#7FBAFF"> ⠿ Container gitea_cont  Removed                                      1.6s</span>
<span style="color:#7FBAFF"> ⠿ Container gitea-db-1  Removed                                      0.4s</span>
<span style="color:#7FBAFF"> ⠿ Network gitea_gitea   Removed                                      0.7s</span>
</pre>

###### Locate desired volumes

```shell
ls /var/lib/docker/volumes/gitea_d* -l
```

<pre>
/var/lib/docker/volumes/gitea_data:
total 4
drwxr-xr-x. 5 root root 4096 Dec  8  2021 <span style="color:#0087FF">_data</span>

/var/lib/docker/volumes/gitea_db:
total 4
drwx------. 19 70 ssh_keys 4096 Jul 19 10:35 <span style="color:#0087FF">_data</span></pre>

On this scenario, we want **BOTH**.




###### Compress Gitea data volumes

```shell
zip -r /tmp/gitea_data.zip /var/lib/docker/volumes/gitea_db /var/lib/docker/volumes/gitea_data
```

This returns some text and can take a while so... wait.

```text
Too much text so imagine a bunch of `adding: var/lib/docker/volumes/gitea_ etc`
```

###### List compressed file

```shell
ls /tmp/gitea_data.zip  -l
```

```text
-rw-r--r--. 1 root root 5486975291 Jul 19 10:59 /tmp/gitea_data.zip
```

###### Send file to the srv.

It can take a while so... wait?

```shell
scp /tmp/gitea_data.zip user@srv:/tmp
```

#### From SRV

##### Unzip to the desired location, maintaining permissions.

> Learn from the mistakes, use the `/` path and let everything sit correctly by itself.

Also, again, can take a while, let it do the needful.

```shell
unzip -X /tmp/gitea_data.zip -d /
```

```text
   creating: /scripts/docker/gitea/
  inflating: /scripts/docker/gitea/app.ini  
  inflating: /scripts/docker/gitea/docker-compose.yml  
   creating: /scripts/docker/gitea/renderers/
 extracting: /scripts/docker/gitea/renderers/.placeholder  
  inflating: /scripts/docker/gitea/Dockerfile
```

##### Start docker service.

```shell
cd /scripts/docker/gitea/ && docker-compose up -d
```

<pre><span style="color:#7FBAFF">[+] Running 20/20</span>
<span style="color:#7FBAFF"> ⠿ db Pulled                                                                                                                                                                                                                           8.5s</span>
<span style="color:#7FBAFF">   ⠿ 4c0d98bf9879 Pull complete                                                                                                                                                                                                        0.6s</span>
<span style="color:#7FBAFF">   ⠿ 7ff5918c11c3 Pull complete                                                                                                                                                                                                        0.7s</span>
<span style="color:#7FBAFF">   ⠿ c393806625cd Pull complete                                                                                                                                                                                                        0.8s</span>
<span style="color:#7FBAFF">   ⠿ 9307f3bcca3a Pull complete                                                                                                                                                                                                        3.8s</span>
<span style="color:#7FBAFF">   ⠿ 5eee78b95230 Pull complete                                                                                                                                                                                                        3.9s</span>
<span style="color:#7FBAFF">   ⠿ c0f2174cad0e Pull complete                                                                                                                                                                                                        3.9s</span>
<span style="color:#7FBAFF">   ⠿ dd6b4e21c993 Pull complete                                                                                                                                                                                                        3.9s</span>
<span style="color:#7FBAFF">   ⠿ 1011823211fa Pull complete                                                                                                                                                                                                        3.9s</span>
<span style="color:#7FBAFF"> ⠿ gitea_cont Pulled                                                                                                                                                                                                                   9.9s</span>
<span style="color:#7FBAFF">   ⠿ 31e352740f53 Already exists                                                                                                                                                                                                       0.0s</span>
<span style="color:#7FBAFF">   ⠿ 395889657acd Pull complete                                                                                                                                                                                                        2.8s</span>
<span style="color:#7FBAFF">   ⠿ fca9ea837ad0 Pull complete                                                                                                                                                                                                        2.8s</span>
<span style="color:#7FBAFF">   ⠿ 27ebbe228e73 Pull complete                                                                                                                                                                                                        2.9s</span>
<span style="color:#7FBAFF">   ⠿ 34a463970b12 Pull complete                                                                                                                                                                                                        4.2s</span>
<span style="color:#7FBAFF">   ⠿ 791a5d24dab3 Pull complete                                                                                                                                                                                                        4.3s</span>
<span style="color:#7FBAFF">   ⠿ ad1a83d556b7 Pull complete                                                                                                                                                                                                        4.3s</span>
<span style="color:#7FBAFF">   ⠿ 8da8fce0c22f Pull complete                                                                                                                                                                                                        5.2s</span>
<span style="color:#7FBAFF">   ⠿ 559bdde1e56a Pull complete                                                                                                                                                                                                        5.2s</span>
<span style="color:#7FBAFF">   ⠿ 40fa068ee382 Pull complete                                                                                                                                                                                                        5.3s</span>
<span style="color:#7FBAFF">[+] Running 5/5</span>
<span style="color:#7FBAFF"> ⠿ Network gitea_gitea   Created                                                                                                                                                                                                       0.2s</span>
<span style="color:#7FBAFF"> ⠿ Volume &quot;gitea_data&quot;   Created                                                                                                                                                                                                       0.0s</span>
<span style="color:#7FBAFF"> ⠿ Volume &quot;gitea_db&quot;     Created                                                                                                                                                                                                       0.0s</span>
<span style="color:#7FBAFF"> ⠿ Container gitea-db-1  Started                                                                                                                                                                                                       1.5s</span>
<span style="color:#7FBAFF"> ⠿ Container gitea_cont  Started                                                                                                                                                                                                       0.6s</span></pre>

```shell
docker-compose logs -f
```

<!--suppress SpellCheckingInspection, SpellCheckingInspection -->
<pre>docker-compose logs -f
<span style="color:#CC3980">gitea-db-1  | </span>
<span style="color:#CC3980">gitea-db-1  | </span>PostgreSQL Database directory appears to contain a database; Skipping initialization
<span style="color:#CC3980">gitea-db-1  | </span>
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.664 UTC [1] LOG:  starting PostgreSQL ???? on x86_64-pc-linux-musl, compiled by gcc (Alpine ????) ???? ?????, 64-bit
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.665 UTC [1] LOG:  listening on IPv4 address &quot;???????&quot;, port 5432
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.665 UTC [1] LOG:  listening on IPv6 address &quot;::&quot;, port 5432
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.669 UTC [1] LOG:  listening on Unix socket &quot;/var/run/postgresql/.s.PGSQL.5432&quot;
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.672 UTC [22] LOG:  database system was shut down at 2023-07-19 10:48:03 UTC
<span style="color:#CC3980">gitea-db-1  | </span>2023-07-19 11:42:58.676 UTC [1] LOG:  database system is ready to accept connections
<span style="color:#7F7FFF">gitea_cont  | </span>Server listening on :: port 22.
<span style="color:#7F7FFF">gitea_cont  | </span>Server listening on 0.0.0.0 port 22.
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:223:runWeb() [I] Starting Gitea on PID: 20
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:148:serveInstalled() [I] Gitea version: 1.20.0 built with GNU Make 4.4.1, go1.20.6 : bindata, timetzdata, sqlite, sqlite_unlock_notify
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:149:serveInstalled() [I] App path: /usr/local/bin/gitea
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:150:serveInstalled() [I] Work path: /data/gitea
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:151:serveInstalled() [I] Custom path: /data/gitea
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:152:serveInstalled() [I] Config file: /data/gitea/conf/app.ini
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:153:serveInstalled() [I] Run mode: prod
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 cmd/web.go:154:serveInstalled() [I] Prepare to run web server
<span style="color:#7F7FFF">gitea_cont  | </span>2023/07/19 13:42:59 routers/init.go:112:InitWebInstalled() [I] Git version: 2.40.1, Wire Protocol Version 2 Enabled (home: /data/gitea/home)
</pre>

#### Traefik

##### Update SRV Traefik to accept traffic for that service.

Configure SRV Traefik to redirect the traffic towards the `Gitea` container.

##### Update Pi4 Traefik to use SRV as backend.

Configure Pi4 Traefik to use as a backend for the `Gitea` service, the host SRV.

##### Test the service is up and running

Test the Gitea service URL

```shell
curl -I https://gitea.filterhome.xyz/
```

Returns a status code `200`, therefore we were able to receive a successful request (still it's recommended to doublecheck through the browser and ensure that the correct service was served, as it could conflict with other rules set, returning a status `200` for a different service than the desired).

```text
HTTP/2 200 
date: Wed, 19 Jul 2023 11:55:15 GMT
permissions-policy: vibrate=('self'); geolocation=('self'); midi=('self'); notifications=('self'); push=('self'); microphone=(); camera=(); magnetometer=(); gyroscope=(); speaker=(); vibrate=('self'); fullscreen=('self')
referrer-policy: same-origin
strict-transport-security: max-age=31536000; includeSubDomains
x-content-type-options: nosniff
x-frame-options: DENY
x-xss-protection: 1; mode=block
```

## Part 2

### Deploy

Since there only can be a single `IstioOperator` kind object per file, I had to spread them in multiple files:

- IstioOperator_IstioConfig.yaml
- IstioOperator_IstioEgress.yaml
- IstioOperator_LocalIngress.yaml
- IstioOperator_PublicIngress.yaml

#### IstioOperator_IstioConfig.yaml

```shell
istioctl install -f P2_Istio_Config/IstioOperator_IstioConfig.yaml
```

<pre>This will install the Istio 1.18.2 minimal profile with [&quot;Istio core&quot; &quot;Istiod&quot;] components into the cluster. Proceed? (y/N) y
<span style="color:#7F3FBF">✔</span> Istio core installed                                                                                                                                                                                                                    
<span style="color:#7F3FBF">✔</span> Istiod installed                                                                                                                                                                                                                        
<span style="color:#7F3FBF">✔</span> Installation complete
Making this installation the default for injection and validation.</pre>

#### IstioOperator_IstioEgress.yaml

```shell
istioctl install -y -f P2_Istio_Config/IstioOperator_IstioEgress.yaml
```

<pre><span style="color:#7F3FBF">✔</span> Egress gateways installed                                                                                                                                                                                                               
<span style="color:#7F3FBF">✔</span> Installation complete </pre>

> **Note**:\
> The egress resource doesn't work "right off the bat", requires some configurations regarding this matter, don't think this will be done on this "walkthrough".

#### IstioOperator_LocalIngress.yaml

```shell
istioctl install -y -f P2_Istio_Config/IstioOperator_LocalIngress.yaml
```

<pre><span style="color:#7F3FBF">✔</span> Ingress gateways installed                                                                                                                                                                                                              
<span style="color:#7F3FBF">✔</span> Installation complete</pre>

#### IstioOperator_PublicIngress.yaml

```shell
istioctl install -y -f P2_Istio_Config/IstioOperator_PublicIngress.yaml
```

<pre><span style="color:#7F3FBF">✔</span> Ingress gateways installed                                                                                                                                                                                                              
<span style="color:#7F3FBF">✔</span> Installation complete    </pre>


### Check resources created exist.


> **Disclaimer:**\
> I use MetalLB to assist on the external IP provisioning for the service(s) from the Kubernetes cluster.


```shell
kubectl get svc -n istio-system -owide -l istio | grep LoadBalancer
```

<pre>
egress-gw              <span style="color:#FF7F7F"><b>LoadBalancer</b></span>   10.106.14.109    192.168.1.90   80:31576/TCP,443:30248/TCP                   2m44s   app=istio-egress-gw,istio=egress-gw
istio-local-ingress    <span style="color:#FF7F7F"><b>LoadBalancer</b></span>   10.107.40.136    192.168.1.81   15021:31546/TCP,80:31242/TCP,443:32271/TCP   2m6s    app=istio-local-ingress,istio=local-ingress
istio-public-ingress   <span style="color:#FF7F7F"><b>LoadBalancer</b></span>   10.100.188.236   192.168.1.80   15021:30125/TCP,80:30999/TCP,443:32596/TCP   100s    app=istio-public-ingress,istio=public-ingress
</pre>


## Part 3

### Deploy the configurations towards the "external" resources/services.

> **Note:**\
> They are external respective to the Kubernetes cluster, not to the "local/home" network.

```shell
kubectl apply -f P3_External_Services -R
```

```text
namespace/external created
destinationrule.networking.istio.io/filebrowser created
serviceentry.networking.istio.io/filebrowser-se created
virtualservice.networking.istio.io/filebrowser-vs created
gateway.networking.istio.io/http-to-https-public created
gateway.networking.istio.io/public-gateway created
destinationrule.networking.istio.io/gitea created
serviceentry.networking.istio.io/gitea-se created
virtualservice.networking.istio.io/gitea-vs created
workloadentry.networking.istio.io/srv-host created
destinationrule.networking.istio.io/jelly created
serviceentry.networking.istio.io/jelly-se created
virtualservice.networking.istio.io/jelly-vs created
destinationrule.networking.istio.io/tube created
serviceentry.networking.istio.io/tube-se created
virtualservice.networking.istio.io/tube-vs created
```

### Deploy the `cert-manager` CRDs.

As per [cer-manager installation guide through `kubectl`](https://cert-manager.io/docs/installation/kubectl/), I ran the following command to install the CRDs:

```shell
kubectl apply -f https://github.com/cert-manager/cert-manager/releases/download/v1.12.0/cert-manager.yaml
```

> **Note**:\
> I installed a while beforehand.

### Verify `cert-manager` installation.

```shell
kubectl get pods --namespace cert-manager
```

```text
NAME                                       READY   STATUS    RESTARTS      AGE
cert-manager-5f68c9c6dd-q7mfk              1/1     Running   1 (33h ago)   7d7h
cert-manager-cainjector-57d6fc9f7d-9c2mp   1/1     Running   1 (33h ago)   7d7h
cert-manager-webhook-5b7ffbdc98-g5z98      1/1     Running   0             7d7h
```

### Set `cert-manager` config to provision LE certificates through ACME HTTP01 Challenge.

```shell
kubectl apply -f P3_Certificate_Manager
```

```text
clusterissuer.cert-manager.io/letsencrypt-public created
certificate.cert-manager.io/filterhome-domain-cert-public created
```

### Monitor the Certificate Provisioning process

#### Check Certificate Issuer status


```shell
kubectl get -n istio-system certificate filterhome-domain-cert-public -o jsonpath='{.metadata.name}{"\t"}{.status.conditions[].reason}{"\t"}{.status.conditions[].message}{"\n"}'
```

```text
filterhome-domain-cert-public	DoesNotExist	Issuing certificate as Secret does not exist
```

Once the certificates are provisioned, it will look like this:

```text
filterhome-domain-cert-public   Ready   Certificate is up to date and has not expired
```


#### Certificate logs

This can take some time, do not despair (yet).

If you notice that it's stuck for some minutes, consider checking the access logs.

```shell
kubectl get events -n istio-system --field-selector involvedObject.name=filterhome-domain-cert-public,involvedObject.kind=Certificate  --sort-by=.metadata.creationTimestamp --watch
```

```text
LAST SEEN   TYPE     REASON    OBJECT                                      MESSAGE
0s          Normal   Issuing   certificate/filterhome-domain-cert-public   Issuing certificate as Secret does not exist
0s          Normal   Generated   certificate/filterhome-domain-cert-public   Stored new private key in temporary Secret resource "filterhome-domain-cert-public-qmnzd"
0s          Normal   Requested   certificate/filterhome-domain-cert-public   Created new CertificateRequest resource "filterhome-domain-cert-public-csfpk"
0s          Normal   Issuing     certificate/filterhome-domain-cert-public   The certificate has been successfully issued
```

#### Access Logs

On the access logs we can see the `ACME Challenge` requests, and that they are being routed towards the service `cm-acme-http-solver-qvpvl.istio-system.svc.cluster.local`.

As well, some of the first entries result in `503 - no_healthy_upstream`, why? Simply because the backend is still waking up, give it a min or two before being afraid if this messages maintain. 

```shell
kubectl logs -l istio=public-ingress -n istio-system -f
```

```text
[2023-07-26T22:22:13.641Z] "GET /.well-known/acme-challenge/abcd1234abcd1234abcd1234abcd1234abcd1234 HTTP/1.1" 503 UH no_healthy_upstream - "-" 0 19 0 - "172.17.247.0" "cert-manager-challenges/v1.12.0 (linux/arm64) cert-manager/abcd1234abcd1234abcd1234abcd1234abcd1234" "987765-asdv-asdf-978987979789" "gitea.filterhome.xyz" "-" outbound|8089||cm-acme-http-solver-qvpvl.istio-system.svc.cluster.local - 172.17.121.127:8080 172.17.247.0:13176 - -
[2023-07-26T22:22:23.538Z] "GET /.well-known/acme-challenge/abcd1234abcd1234abcd1234abcd1234abcd1234 HTTP/1.1" 200 - via_upstream - "-" 0 87 6 3 "172.17.247.0" "cert-manager-challenges/v1.12.0 (linux/arm64) cert-manager/abcd1234abcd1234abcd1234abcd1234abcd1234" "987765-asdv-asdf-978987979789" "gitea.filterhome.xyz" "172.17.121.95:8089" outbound|8089||cm-acme-http-solver-qvpvl.istio-system.svc.cluster.local 172.17.121.127:44994 172.17.121.127:8080 172.17.247.0:57220 - -
```


### Test HTTP to HTTPS redirect gateway


```shell
curl http://gitea.filterhome.xyz -I
```

```text
HTTP/1.1 301 Moved Permanently
location: https://gitea.filterhome.xyz/
date: Wed, 26 Jul 2023 22:34:13 GMT
server: istio-envoy
transfer-encoding: chunked
```


### Test HTTPS access towards services works correctly


```shell
curl gitea.filterhome.xyz -I --follow-redirects
```

```text
HTTP/2 200 
date: Wed, 26 Jul 2023 22:33:53 GMT
x-envoy-upstream-service-time: 6
server: istio-envoy
```

### Test the local Ingress Load Balancer

Good ol' fake headers bypassing.

```shell
curl 192.168.1.81/web/index.html -HHOST:"jelly.filter.home" -I
```

```text
HTTP/1.1 200 OK
accept-ranges: bytes
content-type: text/html
date: Wed, 26 Jul 2023 22:56:13 GMT
etag: "1d975f47df7d992"
last-modified: Sun, 23 Apr 2023 15:01:33 GMT
server: istio-envoy
x-response-time-ms: 1
content-length: 7442
x-envoy-upstream-service-time: 3
```

## Part 4


### Update local DNS

I updated the Local DNS to point towards the new-architecture/new-resources.

### Certificate Generation

> **Note:**\
> Basically I followed [this post from Medium](https://systemweakness.com/create-internal-ssl-certificates-with-cert-manager-851fc886628e). 

First, let's generate a certificate, and it's key.

```shell
openssl req -x509 -newkey rsa:4096 -sha256 -days 5 -nodes \
-keyout ca.filter.home.key -out ca.filter.home.cer \
-subj /C=ES/ST=BAR/O=FilterHome/CN=ca.filter.home \
-extensions ext \
-config <(cat <<EOF
[req]
distinguished_name=req
[ext]
keyUsage=critical,keyCertSign,cRLSign
basicConstraints=critical,CA:true,pathlen:1
subjectAltName=DNS:ca.filter.home
EOF
)
```

```text
cmdsubst heredoc> [req]
cmdsubst heredoc> distinguished_name=req
cmdsubst heredoc> [ext]
cmdsubst heredoc> keyUsage=critical,keyCertSign,cRLSign
cmdsubst heredoc> basicConstraints=critical,CA:true,pathlen:1
cmdsubst heredoc> subjectAltName=DNS:ca.filter.home
cmdsubst heredoc> EOF
cmdsubst> )
........+.+..+............+++++++++++++++++++++++++++++++++++++++++++++*..+...+..........+..+...+.+.....................+.....+.......+..+.+.....+++++++++++++++++++++++++++++++++++++++++++++*.......+.........+.+..+.+.....................+......+.......................+......+......+............+..........+.....+.......+.....+..........+.....+...+...+....+...+........+....+......+.....+...................+.......................+.+.....+......+.+...+......+.....+....+.....+.+...........+....+...+...+............+..+......+.......+..+....+..............+..........+..................+..+.+......+......+........+.+......+.....+............+................+..+....+......+.........+......+........+.+.....+...+.+..+....+...+..+.+.....+...+........................................+.................+.........+................+..+......+.+..............+......+.+.....+...............+++++
.+............+.....+++++++++++++++++++++++++++++++++++++++++++++*........+............+....................+.+..+...+...+.+...+..+..........+...+..+.+.....+.........+...+...+.......+......+..+...+.+......+++++++++++++++++++++++++++++++++++++++++++++*...+...........+.......+......+...............+..+...+.........+..........+...+..+.......+..............+....+...+........+..........+..+...+....+.....+................+..+..........+++++
-----
```

Now we obtain the base64 contents (don't need to store, preferably **don't store the output**).

```shell
cat ca.filter.home.cer | base64 | tr -d '\n' > tls.crt
```

```shell
cat ca.filter.home.key | base64 | tr -d '\n' > tls.key
```


Modify the file `Secret.yaml` and set the output from the files as value from the data fields, each one according to their filename.

```shell
kubectl apply -f P4_Local_Certs/Secret.yaml
```

```text
secret/local-ca created
```

Let's deploy the Issuer service configuration.

```shell
kubectl apply -f P4_Local_CA/Issuer.yaml                                                           
```

```text
clusterissuer.cert-manager.io/ca-issuer created
certificate.cert-manager.io/local-wildcard-certificate created
```

### Monitor Certificate Provisioning process

```shell
kubectl get events -n istio-system --field-selector involvedObject.name=local-wildcard-certificate,involvedObject.kind=Certificate  --sort-by=.metadata.creationTimestamp --watch
```

```text
LAST SEEN   TYPE     REASON      OBJECT                                   MESSAGE
3m48s       Normal   Issuing     certificate/local-wildcard-certificate   Issuing certificate as Secret does not exist
3m43s       Normal   Generated   certificate/local-wildcard-certificate   Stored new private key in temporary Secret resource "local-wildcard-certificate-f7g4f"
3m43s       Normal   Requested   certificate/local-wildcard-certificate   Created new CertificateRequest resource "local-wildcard-certificate-8rndg"
3m42s       Normal   Issuing     certificate/local-wildcard-certificate   The certificate has been successfully issued
```

### Check status of the certificate provisioned.

```shell
kubectl get -n istio-system certificate local-wildcard-certificate -o jsonpath='{.metadata.name}{"\t"}{.status.conditions[].reason}{"\t"}{.status.conditions[].message}{"\n"}'
```

```text
local-wildcard-certificate      Ready   Certificate is up to date and has not expired
```

### Test local Gateway

Now that the certificate is provisioned, let's check the status of the local gateway deployed in the [Part 3](#part-3)

#### HTTP

```shell
curl jelly.filter.home/web/index.html -I
```

```text
HTTP/1.1 200 OK
accept-ranges: bytes
content-type: text/html
date: Thu, 27 Jul 2023 14:18:24 GMT
etag: "1d975f47df7d992"
last-modified: Sun, 23 Apr 2023 15:01:33 GMT
server: istio-envoy
x-response-time-ms: 0
content-length: 7442
x-envoy-upstream-service-time: 2
```

#### HTTPS

```shell
curl https://jelly.filter.home/web/index.html -I -k
```

```text
HTTP/2 200 
accept-ranges: bytes
content-type: text/html
date: Thu, 27 Jul 2023 14:18:59 GMT
etag: "1d975f47df7d992"
last-modified: Sun, 23 Apr 2023 15:01:33 GMT
server: istio-envoy
x-response-time-ms: 0
content-length: 7442
x-envoy-upstream-service-time: 2
```

> **Note:**\
> It's extremely possible that I set HTTP to HTTPS redirect also locally, still need to decide if there is any reason for which I would like to maintain the local HTTP traffic.


### Part 5

I decided to use a SSD with the Pi4 through a USB3.0 connector.

On the Orange Pi5 I intend to use an m.2 that I bought for it, yet need to try if it works correctly.

Therefore, I will remove 1 node from the current Kubernetes cluster and try there the m.2 that I got.

#### Remove 1 OrangePI Node

https://stackoverflow.com/questions/35757620/how-to-gracefully-remove-a-node-from-kubernetes

```shell
kubecl get nodes
```

```text
NAME                  STATUS   ROLES           AGE    VERSION
masterk.filter.home   Ready    control-plane   114d   v1.26.2
slave01.filter.home   Ready    <none>          114d   v1.26.2
```

```shell
kubectl drain slave01.filter.home
```

```text
node/slave01.filter.home cordoned
error: unable to drain node "slave01.filter.home" due to error:[cannot delete Pods with local storage (use --delete-emptydir-data to override): default/duckdns-dpl-856cc44f65-79mtn, default/helloworld-6798765f88-ql26n, default/helloworld-nginx-6bb45bc45c-5z7r4, default/helloworld2-dc9cb5db6-m47x7, istio-system/egress-gw-c857495b4-zzlkp, istio-system/istio-local-ingress-778f675879-w9mdh, istio-system/istio-public-ingress-5bb994c8b7-j9hgr, cannot delete DaemonSet-managed Pods (use --ignore-daemonsets to ignore): kube-system/calico-node-sv6sp, kube-system/kube-proxy-jpnkt, metallb-system/speaker-9p4fz], continuing command...
There are pending nodes to be drained:
 slave01.filter.home
cannot delete Pods with local storage (use --delete-emptydir-data to override): default/duckdns-dpl-856cc44f65-79mtn, default/helloworld-6798765f88-ql26n, default/helloworld-nginx-6bb45bc45c-5z7r4, default/helloworld2-dc9cb5db6-m47x7, istio-system/egress-gw-c857495b4-zzlkp, istio-system/istio-local-ingress-778f675879-w9mdh, istio-system/istio-public-ingress-5bb994c8b7-j9hgr
cannot delete DaemonSet-managed Pods (use --ignore-daemonsets to ignore): kube-system/calico-node-sv6sp, kube-system/kube-proxy-jpnkt, metallb-system/speaker-9p4fz
```


```shell
kubectl drain slave01.filter.home --delete-emptydir-data --ignore-daemonsets
```

```text
node/slave01.filter.home already cordoned
Warning: ignoring DaemonSet-managed Pods: kube-system/calico-node-sv6sp, kube-system/kube-proxy-jpnkt, metallb-system/speaker-9p4fz
evicting pod metallb-system/controller-bdf98b979-zqh7n
evicting pod default/helloworld-6798765f88-ql26n
evicting pod default/helloworld-nginx-6bb45bc45c-5z7r4
evicting pod default/helloworld2-dc9cb5db6-m47x7
evicting pod istio-system/istio-local-ingress-778f675879-w9mdh
evicting pod istio-system/egress-gw-c857495b4-zzlkp
evicting pod cert-manager/cert-manager-5f68c9c6dd-q7mfk
evicting pod cert-manager/cert-manager-webhook-5b7ffbdc98-g5z98
evicting pod cert-manager/cert-manager-cainjector-57d6fc9f7d-9c2mp
evicting pod default/duckdns-dpl-856cc44f65-79mtn
evicting pod istio-system/istio-public-ingress-5bb994c8b7-j9hgr
error when evicting pods/"istio-local-ingress-778f675879-w9mdh" -n "istio-system" (will retry after 5s): Cannot evict pod as it would violate the pod's disruption budget.
error when evicting pods/"egress-gw-c857495b4-zzlkp" -n "istio-system" (will retry after 5s): Cannot evict pod as it would violate the pod's disruption budget.
error when evicting pods/"istio-public-ingress-5bb994c8b7-j9hgr" -n "istio-system" (will retry after 5s): Cannot evict pod as it would violate the pod's disruption budget.
```

Well, it's on it, so let's open a new shell.


#### Cannot evict pod as it would violate the pod's disruption budget.

Let's delete the "remaining" pods manually.

```shell
kubectl get pods -A -owide | grep slave
istio-system     egress-gw-c857495b4-zzlkp                     1/1     Running   0                  25h     172.17.247.6     slave01.filter.home   <none>           <none>
istio-system     istio-local-ingress-778f675879-w9mdh          1/1     Running   0                  25h     172.17.247.10    slave01.filter.home   <none>           <none>
istio-system     istio-public-ingress-5bb994c8b7-j9hgr         1/1     Running   0                  25h     172.17.247.19    slave01.filter.home   <none>           <none>
kube-system      calico-node-sv6sp                             1/1     Running   6 (2d17h ago)      114d    192.168.1.11     slave01.filter.home   <none>           <none>
kube-system      kube-proxy-jpnkt                              1/1     Running   6 (2d17h ago)      114d    192.168.1.11     slave01.filter.home   <none>           <none>
metallb-system   speaker-9p4fz                                 1/1     Running   9 (2d17h ago)      114d    192.168.1.11     slave01.filter.home   <none>           <none>
```


```shell
kubectl delete pod -n istio-system egress-gw-c857495b4-zzlkp
```

```text
pod "egress-gw-c857495b4-zzlkp" deleted
```

```shell
kubectl get pods -A -owide
```

````text
NAMESPACE        NAME                                          READY   STATUS    RESTARTS           AGE     IP               NODE                  NOMINATED NODE   READINESS GATES
...
istio-system     egress-gw-c857495b4-ll9jr                     1/1     Running   0                  2m15s   172.17.121.119   masterk.filter.home   <none>           <none>
...
````


Alright, it worked correctly, let's automate it a little.

```shell
kubectl get pods -A -owide | grep slave | awk '{ printf "kubectl delete pod -n %s %s\n",$1,$2 }'
```

```text
kubectl delete pod -n istio-system istio-local-ingress-778f675879-w9mdh
kubectl delete pod -n istio-system istio-public-ingress-5bb994c8b7-j9hgr
kubectl delete pod -n kube-system calico-node-sv6sp
kubectl delete pod -n kube-system kube-proxy-jpnkt
kubectl delete pod -n metallb-system speaker-9p4fz
```

```shell
kubectl get pods -A -owide | grep slave | awk '{ printf "kubectl delete pod -n %s %s\n",$1,$2 }' | bash
```

```text
pod "istio-local-ingress-778f675879-w9mdh" deleted
pod "istio-public-ingress-5bb994c8b7-j9hgr" deleted
pod "calico-node-sv6sp" deleted
pod "kube-proxy-jpnkt" deleted
pod "speaker-9p4fz" deleted
```

Well, finally things where drained, with a little of "help".

```text
pod/istio-local-ingress-778f675879-w9mdh evicted
pod/istio-public-ingress-5bb994c8b7-j9hgr evicted
node/slave01.filter.home drained
```

```shell
kubectl delete node slave01.filter.home
```

```text
node "slave01.filter.home" deleted
```

### Part 6




# I am here <----


```shell

```




---













## Difficulties



The Certificate Manager must be located at the same namespace as the `istiod` service, without the LB location being taken into account.


## no healthy upstream

During the "testing of configurations", I created a the `DestinationRule` for the Jelly on the namespace `default`, instead of `external`.

*In theory,* I did cleanup everything before going to sleep, to avoid, u know, "issues".

On this scenario, seems like I set the `DR` (DestinationRule) with the external namespace, then when I did a general cleanup (`kubectl delete -f P3etc`), the resource wasn't deleted, and seems like I overlooked it.

### How did the troubleshooting process go?

#### Check yaml

First of all, checking the `.yaml` files that I created on the `/external` namespace.

Also used the `diff` command to display the difference on the lines between the services that worked, and the `jelly` Istio resources.

Everything looked correct.

#### Check the destination server.

I checked the logs on the external resource reverse proxy, which managed the ingress traffic and forwards it into the respective container.

What did I find:

- **NOTHING**

Not a single trace of traffic when I was accessing through `jelly.filterhome.xyz`.

Weird right?

What happens when I try to access `gitea.filterhome.xyz`?

- I get into the service without any issue through the browser.
- There is traffic logs

Conclusion?

<ins>**The traffic doesn't reach the destination server.**</ins>

#### Using mock services.

I have modified the `Gitea` service, adding a `HOST` header to artificially match the external's server reverse proxy rules.

```yaml
http:
  route:
    - destination:
          headers:
            request:
              set:
                HOST: "jelly.filterhome.xyz"
...
```

> **Note:**\
> Didn't require to modify anything else since the request where all going to the same destination server.

### Istio cluster service fqdn

```shell
istioctl proxy-config cluster -n istio-system istio-public-ingress-85b86998fc-spksq  | grep filterhome.xyz
```

<pre>filebrowser.<span style="color:#FF7F7F"><b>filterhome.xyz</b></span>                                443       -          outbound      EDS            filebrowser.external
gitea.<span style="color:#FF7F7F"><b>filterhome.xyz</b></span>                                      443       -          outbound      EDS            gitea.external
jelly.<span style="color:#FF7F7F"><b>filterhome.xyz</b></span>                                      443       -          outbound      EDS            jelly.default
tube.<span style="color:#FF7F7F"><b>filterhome.xyz</b></span>                                       443       -          outbound      EDS            tube.external</pre>

Alright, we got the output, but why does it say `jelly.default`? All the other entries maintain the format `$SERVICE.external`, could it be ...?

```shell
kubectl get dr -n default 
```


```text
NAME    HOST                   AGE
jelly   jelly.filterhome.xyz   7h32m
```

Effectively, I did miss removing that service. But... if I did miss that, **what else** did I miss?

I should check for the following istio objects then:

- DestinationRule
- VirtualService
- ServiceEntry

```shell
kubectl get dr,vs,se -n default
```

```text
NAME                                        HOST                   AGE
destinationrule.networking.istio.io/jelly   jelly.filterhome.xyz   7h35m

NAME                                        HOSTS                                                                         LOCATION        RESOLUTION   AGE
serviceentry.networking.istio.io/jelly-se   ["jelly.filterhome.xyz","jelly.filterhome.duckdns.org","jelly.filter.home"]   MESH_INTERNAL   NONE         7h35m
```

At the end turns out I didn't delete this both resources (DestinationRule and ServiceEntry).

This called for a renaming of the configuration, so this doesn't occur _again_ (until occurs).

---

## 2X1 Able to get `staging` LE certs, but not the `production` one when using custom Istio selector, and only able to get `production` LE certs when using the default `istio: ingressgateway` selector.

Did _some_ ~~lots of~~ tests in order to determine what was happening, since it was me trying to figure out what it was, I didn't really documented a lot. 

This issue popped out 2 questions:

- Which is the difference between LE `staging` and `production`?
- Which is the difference between the default `istio: ingressgateway` and `istio: whateveriwannaputhere`.

### Which is the difference between LE `staging` and `production`?

As far I understood by reading through `The Internet`, the `staging` environment is intended for testing (duh), therefore is a lot more generous when performing the validations.

How much generous? I suspect a whole a lot bunch.

### Which is the difference between LE `staging` and `production`?

Let's check the "access logs" differences when using one selector or another. 

> **Note**:\
> The setting `meshConfig.accessLogFile` was set to `/dev/stdout` in the Istio configuration during the installation of such, it's required to capture the Istio deployment access logs.\
> Therefore, in case you don't see any activity, consider checking for this.

#### Custom

```shell
kubectl get logs -l istio=public-ingress -n istio-system
```

```text
[2023-07-26T17:52:19.189Z] "GET /.well-known/acme-challenge/abcd123412341234-12 HTTP/1.1" 404 - via_upstream - "-" 0 153 6 4 "172.17.121.64" "cert-manager-challenges/v1.12.0 (linux/arm64) cert-manager/bd192c4f76dd883f9ee908035b894ffb49002384" "25dd1fdc-fad0-4d50-8acc-fd347374a5d1" "pur.filterhome.xyz" "172.17.247.5:80" outbound|80||hell.default.svc.cluster.local 172.17.247.28:33176 172.17.247.28:8080 172.17.121.64:4678 - -
```

#### default ingressgateway

```shell
kubectl get logs -l istio=ingressgateway -n istio-system
```

```text
[2023-07-26T17:57:19.789Z] "GET /.well-known/acme-challenge/abcd123412341234abcd123412341234 HTTP/1.1" 503 UH no_healthy_upstream - "-" 0 19 0 - "192.168.1.10" "cert-manager-challenges/v1.12.0 (linux/arm64) cert-manager/bd192c4f76dd883f9ee908035b894ffb49002384" "c968e808-1e51-4394-a50e-dbd90ead0b6e" "tr.filterhome.xyz" "-" outbound|8089||cm-acme-http-solver-tqdlw.istio-system.svc.cluster.local - 172.17.121.109:8080 192.168.1.10:2244 - -
```

> **Note:**\
> Hey you, I am very sure that you read the logs (as it should be), you might was well have noticed that the second one, which I claim to be the one setting that addressed the issue, is actually returning a status code 503.\
> Basically the destination service was booting up, nothing more.\
> Once it was up it allowed the correct provisioning of the certificate(s). 


#### Conclusion

Yes, that's too much text to read something straight up, but first, we need to understand the requirements to pass the LE certificate provisioning:

- The request must return a `200` status code.
- The response received but contain a very specific body, which will be generated based on the acme-challenge key used.

Regarding the 2 `access-log` logs shared above, the main thing I wanted to point out, is the backend service.

- When using a custom selector, the service is the expected backend service, which on this scenario is `hell.default.svc.cluster.local`.

- When using the `ingressgateway` selector, the destination service is `cm-acme-http-solver-tqdlw.istio-system.svc.cluster.local`.

Where does the service `cm-acme-http-solver-tqdlw.istio-system.svc.cluster.local` come from? It's important **because it's the service that should reply to the requests from the ACME HTTP01 challenge**.

Meanwhile, I was using the custom selector, I used the following command:

```shell
kubectl exec "$(kubectl get pod -l app=istiod -o jsonpath={.items..metadata.name} -n istio-system)" -n istio-system curl localhost:8080/debug/configz | jq | less
```

Through the output, I could see an Istio gateway object being listed, with the name `cm-acme-http-solver-ww9t7-istio-autogenerated-k8s-ingress-istio-system`, and as well casually it had the selector set to `istio: ingressgateway`.

That was an advancement, yet I am still confused regarding why is it choosing `istio: ingressgateway`, who is creating this resource, and on top of that, why can't I seem to be able to select the selector manually.

As well, the `creationTimestamp` field was set to `null`, for a reason I wasn't able to notice the existence of this resource previously, because it wasn't created. 
Therefore, I understand that it's a resource that creates spontaneously and does not go through the full process of creating a resource on Istio.

So far I only could consider 2 locations that could be related to this and should allow me for some alteration of the fields from the `Gateway` resource located.

- The Certificate Manager `ClusterIssuer` configurations.
- Isto's MeshConfig configuration.

Long story short, I didn't found anything on the Certificate Manager.

Regarding Istio MeshConfig, what did I do?

Very simple, opened up the [following documentation regarding `Global Mesh Options.MeshConfig`](https://istio.io/latest/docs/reference/config/istio.mesh.v1alpha1/), pressed `Control+F` and searched for `ingressgateway`.

I was able to find the following highlighter fields:
- ingressService
- ingressSelector

Therefore, what did I do?

Update those fields through the usage of an `IstioOperator` config file, to match the resource I was using to accept incoming traffic.

Guess what happened once I updated the fields: 

- ingressService
- ingressSelector

It worked.

ACME challenge was returning a 200 status code.

This left me some questions, especially **how many things can be affected by this behavior**.

As well the hassle of this issue, provided more reasons to use the `ACME DNS01` challenge instead of the `ACME HTTP01`, which will be configured as soon as available.  


### Jellyfin and HTTP2 upgrading